﻿Imports Charmed.Sample.Models
Imports System.IO
Imports System.Linq
Imports System.Net
Imports System.ServiceModel.Syndication
Imports System.Text
Imports System.Text.RegularExpressions
Imports System.Threading.Tasks
Imports System.Xml

Namespace Charmed.Sample.Services
	Public NotInheritable Class WP8RssFeedService
		Inherits RssFeedService
		Public Sub New(ByVal settings As ISettings)
			MyBase.New(settings)
		End Sub

		Protected Overrides Function GetFeedAsync(ByVal feedUriString As String) As Task(Of Models.FeedData)
			Dim task = New TaskCompletionSource(Of Models.FeedData)()

			Dim webClient = New WebClient()
            AddHandler webClient.DownloadStringCompleted, Sub(sender, args)
                                                              If args.Error Is Nothing Then
                                                                  Dim stringReader = New StringReader(args.Result)
                                                                  Dim feedData = New FeedData()
                                                                  Using XmlReader As XmlReader = XmlReader.Create(stringReader)
                                                                      Dim feed = SyndicationFeed.Load(XmlReader)

                                                                      If feed.Title IsNot Nothing AndAlso feed.Title.Text IsNot Nothing Then
                                                                          feedData.Title = feed.Title.Text
                                                                      End If
                                                                      If feed.Description IsNot Nothing AndAlso feed.Description.Text IsNot Nothing Then
                                                                          feedData.Description = feed.Description.Text
                                                                      End If
                                                                      If feed.Items IsNot Nothing AndAlso feed.Items.Any Then
                                                                          feedData.PublishedDate = feed.Items.First.PublishDate.DateTime

                                                                          For Each item In feed.Items
                                                                              Dim feedItem = New FeedItem()
                                                                              If item.Title IsNot Nothing AndAlso item.Title.Text IsNot Nothing Then
                                                                                  feedItem.Title = item.Title.Text
                                                                              End If
                                                                              If item.PublishDate <> Nothing Then
                                                                                  feedItem.PublishedDate = item.PublishDate.DateTime
                                                                              End If
                                                                              If item.Authors IsNot Nothing AndAlso item.Authors.Count > 0 Then
                                                                                  feedItem.PublishedDate = item.PublishDate.DateTime
                                                                              End If
                                                                              If item.Content IsNot Nothing AndAlso (CType(item.Content, TextSyndicationContent)).Text IsNot Nothing Then
                                                                                  feedItem.Content = (CType(item.Content, TextSyndicationContent)).Text
                                                                              ElseIf item.Summary IsNot Nothing AndAlso item.Summary.Text IsNot Nothing Then
                                                                                  feedItem.Content = item.Summary.Text
                                                                              End If
                                                                              If item.Links IsNot Nothing AndAlso item.Links.Count > 0 Then
                                                                                  feedItem.Link = item.Links(0).Uri
                                                                              End If

                                                                              feedItem.Id = Regex.Replace(feedItem.Link.ToString, "[^0-9a-zA-Z\.]+", String.Empty).GetHashCode()

                                                                              feedData.Items.Add(feedItem)
                                                                          Next item
                                                                      End If
                                                                  End Using
                                                                  task.TrySetResult(feedData)
                                                              Else
                                                                  task.TrySetException(args.Error)
                                                              End If
                                                          End Sub
			webClient.DownloadStringAsync(New Uri(feedUriString))

			Return task.Task
		End Function
	End Class
End Namespace
