﻿Imports Charmed.ApplicationBar
Imports Charmed.Sample.Models
Imports System.Collections.ObjectModel
Imports System.Threading.Tasks

Namespace Charmed.Sample.ViewModels
	Public NotInheritable Class WP8FeedItemViewModel
		Inherits FeedItemViewModel
		Public Sub New(ByVal serializer As ISerializer, ByVal storage As IStorage, ByVal secondaryPinner As ISecondaryPinner)
			MyBase.New(serializer, storage, secondaryPinner)
            Me.AppBarButtons = New ObservableCollection(Of BindableApplicationBarIconButton)()

			Me.PinButton = New BindableApplicationBarIconButton With {.IconUri = New Uri("/Assets/PinIcon.png", UriKind.Relative), .Text = "pin", .ClickMethodName = "Pin"}

			Me.UnpinButton = New BindableApplicationBarIconButton With {.IconUri = New Uri("/Assets/UnpinIcon.png", UriKind.Relative), .Text = "unpin", .ClickMethodName = "Unpin"}
		End Sub

		Public Overrides Sub LoadState(ByVal navigationParameter As FeedItem, ByVal pageState As Dictionary(Of String, Object))
			MyBase.LoadState(navigationParameter, pageState)

			UpdateAppBarButtons()
		End Sub

		Public async Function Pin() As Task
			' Pin the feed item, then save it locally to make sure it is still available
			' when they return.
            Dim tileInfo = New TileInfo(Me.FormatTileIdUrl, Me.FeedItem.Title, Constants.AppName, New Uri("/Assets/ApplicationIcon.png", UriKind.Relative))

			Me.IsFeedItemPinned = await Me.secondaryPinner.Pin(tileInfo)

			If Me.IsFeedItemPinned Then
                Await Me.SavePinnedFeedItem
			End If

			UpdateAppBarButtons()
		End Function

		Public async Function Unpin() As Task
			' Unpin, then delete the feed item locally.
            Dim tileInfo = New TileInfo(Me.FormatTileIdUrl)
            Me.IsFeedItemPinned = Not Await Me.secondaryPinner.Unpin(tileInfo)

			If Not Me.IsFeedItemPinned Then
                Await Me.RemovePinnedFeedItem
			End If

			UpdateAppBarButtons()
		End Function

		Private Function FormatTileIdUrl() As String
			Dim queryString = String.Format("parameter={0}", FeedItem.Id)
			Return String.Format(Constants.SecondaryUriFormat, queryString)
		End Function

		Private Sub UpdateAppBarButtons()
			Me.AppBarButtons.Clear()
            If Me.secondaryPinner.IsPinned(Me.FormatTileIdUrl) Then
                Me.AppBarButtons.Add(Me.UnpinButton)
            Else
                Me.AppBarButtons.Add(Me.PinButton)
            End If
		End Sub

		Private privateAppBarButtons As ObservableCollection(Of BindableApplicationBarIconButton)
        Public Property AppBarButtons As ObservableCollection(Of BindableApplicationBarIconButton)
            Get
                Return privateAppBarButtons
            End Get
            Private Set(ByVal value As ObservableCollection(Of BindableApplicationBarIconButton))
                privateAppBarButtons = value
            End Set
        End Property

        Private Property PinButton As BindableApplicationBarIconButton
        Private Property UnpinButton As BindableApplicationBarIconButton
	End Class
End Namespace
