﻿Imports Charmed.Sample.Messages
Imports Charmed.Sample.Models
Imports Charmed.Sample.Tests.Mocks
Imports Charmed.Sample.ViewModels
Imports Microsoft.VisualStudio.TestPlatform.UnitTestFramework
Imports System
Imports System.Collections.Generic

Namespace Charmed.Sample.Tests.ViewModels
    <TestClass>
    Public Class MainViewModelTests
        Private Property RssFeedService As RssFeedServiceMock
        Private Property Navigator As NavigatorMock
        Private Property MessageBus As MessageBusMock

        Private Function GetViewModel() As MainViewModel
            Return New MainViewModel(Me.RssFeedService, Me.Navigator, Me.MessageBus)
        End Function

        <TestInitialize>
        Public Sub Init()
            Me.RssFeedService = New RssFeedServiceMock()
            Me.Navigator = New NavigatorMock()
            Me.MessageBus = New MessageBusMock()
        End Sub

        <TestMethod>
        Public Sub ViewFeed()
            ' Arrange
            Dim viewModel As MainViewModel = Me.GetViewModel

            Dim expectedFeedItem As FeedItem = New FeedItem()

            Dim actualViewModelType As Type = Nothing
            Dim actualFeedItem As FeedItem = Nothing
            Me.Navigator.NavigateToViewModelDelegate = Sub(viewModelType, parameter)
                                                           actualViewModelType = viewModelType
                                                           actualFeedItem = TryCast(parameter, FeedItem)
                                                       End Sub

            ' Act
            viewModel.ViewFeed(expectedFeedItem)

            ' Assert
            Assert.AreSame(expectedFeedItem, actualFeedItem, "FeedItem")
            Assert.AreEqual(GetType(FeedItemViewModel), actualViewModelType, "ViewModel Type")
        End Sub

        <TestMethod>
        Public Sub FeedData()
            ' Arrange
            Dim viewModel As MainViewModel = GetViewModel()

            Dim expectedFeedData As List(Of FeedData) = New List(Of FeedData)()

            Me.RssFeedService.GetFeedsAsyncDelegate = Function() expectedFeedData

            ' Act
            Dim actualFeedData As List(Of FeedData) = viewModel.FeedData

            ' Assert
            Assert.AreSame(expectedFeedData, actualFeedData)
        End Sub

        <TestMethod>
        Public Sub HandleFeedsChangedMessage()
            ' Arrange
            Dim handler As Action(Of FeedsChangedMessage) = Nothing
            Me.MessageBus.SubscribeDelegate = Sub(h) handler = CType(h, Action(Of FeedsChangedMessage))

            Dim viewModel As MainViewModel = GetViewModel()

            Dim expectedFeedData As List(Of FeedData) = New List(Of FeedData)()

            Me.RssFeedService.GetFeedsAsyncDelegate = Function() expectedFeedData

            ' Act
            handler(New FeedsChangedMessage())

            ' Assert
            Assert.AreSame(expectedFeedData, viewModel.FeedData)
        End Sub
    End Class
End Namespace
