﻿Imports Charmed.Sample.Messages
Imports Charmed.Sample.Tests.Mocks
Imports Charmed.Sample.ViewModels
Imports Microsoft.VisualStudio.TestPlatform.UnitTestFramework
Imports System.ComponentModel
Imports System.Collections.Generic
Imports System

Namespace Charmed.Sample.Tests.ViewModels
    <TestClass>
    Public Class SettingsViewModelTests
        Private Property Settings As SettingsMock
        Private Property MessageBox As MessageBusMock

        Private Function GetViewModel() As SettingsViewModel
            Return New SettingsViewModel(Me.Settings, Me.MessageBox)
        End Function

        Private Property PropertiesChanged As List(Of String)
        Private Sub OnPropertyChanged(ByVal sender As Object, ByVal args As PropertyChangedEventArgs)
            PropertiesChanged.Add(args.PropertyName)
        End Sub

        <TestInitialize>
        Public Sub Init()
            Me.Settings = New SettingsMock()
            Me.MessageBox = New MessageBusMock()

            Me.PropertiesChanged = New List(Of String)
        End Sub

        <TestMethod>
        Public Sub InitiallyLoadFeeds()
            ' Arrange
            Dim feeds() As String = {Guid.NewGuid().ToString}

            Me.Settings.TryGetValueDelegate = Function(key, value) feeds

            ' Act
            Dim viewModel As SettingsViewModel = GetViewModel()

            ' Assert
            Assert.AreEqual(feeds(0), viewModel.Feeds(0))
        End Sub

        <TestMethod>
        Public Sub NewFeed()
            ' Arrange
            Dim viewModel As SettingsViewModel = GetViewModel()

            Dim expectedNewFeed As String = Guid.NewGuid.ToString

            ' Act
            AddHandler viewModel.PropertyChanged, AddressOf OnPropertyChanged
            viewModel.NewFeed = expectedNewFeed
            RemoveHandler viewModel.PropertyChanged, AddressOf OnPropertyChanged

            ' Assert
            Assert.AreEqual(expectedNewFeed, viewModel.NewFeed, "NewFeed")
            Assert.IsTrue(Me.PropertiesChanged.Contains("NewFeed"), "Property Changed")
        End Sub

        <TestMethod>
        Public Sub AddFeed()
            ' Arrange
            Dim viewModel As SettingsViewModel = GetViewModel()

            Dim expectedNewFeed As String = Guid.NewGuid.ToString
            viewModel.NewFeed = expectedNewFeed

            Dim wasSettingsUpdated As Boolean = False
            Me.Settings.AddOrUpdateDelegate = Sub(key, value) wasSettingsUpdated = True

            Dim wasMessagePublished As Boolean = False
            Me.MessageBox.PublishDelegate = Sub(m) wasMessagePublished = TypeOf m Is FeedsChangedMessage

            ' Act
            viewModel.AddFeed()

            ' Assert
            Assert.IsTrue(viewModel.Feeds.Contains(expectedNewFeed), "Feeds")
            Assert.AreEqual(String.Empty, viewModel.NewFeed, "NewFeed")
            Assert.IsTrue(wasSettingsUpdated, "Was Settings Updated")
            Assert.IsTrue(wasMessagePublished, "Was Message Published")
        End Sub

        <TestMethod>
        Public Sub RemoveFeed()
            ' Arrange
            Dim viewModel As SettingsViewModel = GetViewModel()

            Dim expectedNewFeed As String = Guid.NewGuid.ToString
            viewModel.NewFeed = expectedNewFeed
            viewModel.AddFeed()

            Dim wasSettingsUpdated As Boolean = False
            Me.Settings.AddOrUpdateDelegate = Sub(key, value) wasSettingsUpdated = True

            Dim wasMessagePublished As Boolean = False
            Me.MessageBox.PublishDelegate = Sub(m) wasMessagePublished = TypeOf m Is FeedsChangedMessage

            ' Act
            viewModel.RemoveFeed(expectedNewFeed)

            ' Assert
            Assert.IsFalse(viewModel.Feeds.Contains(expectedNewFeed), "Feeds")
            Assert.AreEqual(String.Empty, viewModel.NewFeed, "NewFeed")
            Assert.IsTrue(wasSettingsUpdated, "Was Settings Updated")
            Assert.IsTrue(wasMessagePublished, "Was Message Published")
        End Sub
    End Class
End Namespace
