﻿Imports Charmed.Sample.Models
Imports Charmed.Sample.Tests.Mocks
Imports Charmed.Sample.ViewModels
Imports Microsoft.VisualStudio.TestPlatform.UnitTestFramework
Imports Windows.UI.ApplicationSettings
Imports System.Collections.Generic

Namespace Charmed.Sample.Tests.ViewModels
    <TestClass>
    Public Class ShellViewModelTests
        Private Property SettingsManager() As SettingsManagerMock
        Private Property Settings() As SettingsMock

        Private Function GetViewModel() As ShellViewModel
            Return New ShellViewModel(Me.SettingsManager, Me.Settings)
        End Function

        <TestInitialize>
        Public Sub Init()
            Me.SettingsManager = New SettingsManagerMock()
            Me.Settings = New SettingsMock
        End Sub

        <TestMethod>
        Public Sub Initialize()
            ' Arrange
            Dim viewModel As ShellViewModel = Me.GetViewModel

            Dim wasSettingsManagerInitialized As Boolean = False
            Me.SettingsManager.InitializeDelegate = Sub() wasSettingsManagerInitialized = True

            Dim feeds() As String = Nothing
            Me.Settings.AddOrUpdateDelegate = Sub(key, value)
                                                  If key = Constants.FeedsKey Then
                                                      feeds = TryCast(value, String())
                                                  End If
                                              End Sub

            ' Act
            viewModel.Initialize()

            ' Assert
            Assert.IsTrue(wasSettingsManagerInitialized, "Was Settings Manager Initialized")
            Assert.IsTrue(feeds.Length > 0, "Has Some Feeds")
        End Sub

        <TestMethod>
        Public Sub Cleanup()
            ' Arrange
            Dim viewModel As ShellViewModel = Me.GetViewModel

            Dim wasSettingsManagerCleanedUp As Boolean = False
            Me.SettingsManager.CleanupDelegate = Sub() wasSettingsManagerCleanedUp = True

            ' Act
            viewModel.Cleanup()

            ' Assert
            Assert.IsTrue(wasSettingsManagerCleanedUp, "Was Settings Manager Cleaned Up")
        End Sub

        <TestMethod>
        Public Sub OnSettingsRequested()
            ' Arrange
            Dim viewModel As ShellViewModel = Me.GetViewModel

            Dim commands As List(Of SettingsCommand) = New List(Of SettingsCommand)

            viewModel.Initialize()

            ' Act
            Me.SettingsManager.OnSettingsRequested()(commands)

            ' Assert
            Assert.AreEqual(1, commands.Count)
        End Sub
    End Class
End Namespace
