﻿Imports Charmed.Sample.Models
Imports System.Text.RegularExpressions
Imports System.Threading.Tasks
Imports Windows.Web.Syndication

Namespace Charmed.Sample.Services
	''' <summary>
	''' Service to read RSS feeds.
	''' </summary>
	''' <remarks>
	''' The idea behind this class is mostly borrowed from Windows Store Blog Reader Sample
	''' http://msdn.microsoft.com/en-us/library/windows/apps/br211380.aspx
	''' </remarks>
	Public NotInheritable Class Win8RssFeedService
		Inherits RssFeedService
		Public Sub New(ByVal settings As ISettings)
			MyBase.New(settings)
		End Sub

		Protected Overrides async Function GetFeedAsync(ByVal feedUriString As String) As Task(Of Models.FeedData)
            Dim client As SyndicationClient = New SyndicationClient
            Dim feedUri As Uri = Nothing
			If Not Uri.TryCreate(feedUriString, UriKind.RelativeOrAbsolute, feedUri) Then
				Return Nothing
			End If

			Try
                Dim feed As SyndicationFeed = Await client.RetrieveFeedAsync(feedUri)
                Dim feedData As FeedData = New FeedData
				If feed.Title IsNot Nothing AndAlso feed.Title.Text IsNot Nothing Then
					feedData.Title = feed.Title.Text
				End If
				If feed.Subtitle IsNot Nothing AndAlso feed.Subtitle.Text IsNot Nothing Then
					feedData.Description = feed.Subtitle.Text
				End If
				If feed.Items IsNot Nothing AndAlso feed.Items.Count > 0 Then
					feedData.PublishedDate = feed.Items(0).PublishedDate.DateTime
                    Dim item As SyndicationItem
					For Each item In feed.Items
                        Dim feedItem As FeedItem = New FeedItem
                        If item.Title IsNot Nothing AndAlso item.Title.Text IsNot Nothing Then
                            feedItem.Title = item.Title.Text
                        End If
                        If CObj(item.PublishedDate) IsNot Nothing Then
                            feedItem.PublishedDate = item.PublishedDate.DateTime
                        End If
						If item.Authors IsNot Nothing AndAlso item.Authors.Count > 0 Then
							feedItem.PublishedDate = item.PublishedDate.DateTime
						End If
						If feed.SourceFormat = SyndicationFormat.Atom10 Then
							If item.Content IsNot Nothing AndAlso item.Content.Text IsNot Nothing Then
								feedItem.Content = item.Content.Text
							End If
							If item.Id IsNot Nothing Then
								feedItem.Link = New Uri(item.Id)
							End If
						ElseIf feed.SourceFormat = SyndicationFormat.Rss20 Then
							If item.Summary IsNot Nothing AndAlso item.Summary.Text IsNot Nothing Then
								feedItem.Content = item.Summary.Text
							End If
							If item.Links IsNot Nothing AndAlso item.Links.Count > 0 Then
								feedItem.Link = item.Links(0).Uri
							End If
						End If

                        feedItem.Id = Regex.Replace(feedItem.Link.ToString, "[^0-9a-zA-Z\.]+", String.Empty).GetHashCode

						feedData.Items.Add(feedItem)
					Next item
				End If

				Return feedData
            Catch ex As Exception
                Debug.WriteLine(ex.Message)
                Return Nothing
            End Try

		End Function
	End Class
End Namespace
