﻿Imports Charmed.Sample.Models
Imports System.Linq
Imports System.Threading.Tasks

Namespace Charmed.Sample.ViewModels
	Public MustInherit Class FeedItemViewModel
		Inherits ViewModelBase(Of FeedItem)
		Private ReadOnly storage As IStorage
		Protected ReadOnly secondaryPinner As ISecondaryPinner

		Public Sub New(ByVal serializer As ISerializer, ByVal storage As IStorage, ByVal secondaryPinner As ISecondaryPinner)
			MyBase.New(serializer)
			Me.storage = storage
			Me.secondaryPinner = secondaryPinner
		End Sub

		Public Overrides Sub LoadState(ByVal navigationParameter As FeedItem, ByVal pageState As Dictionary(Of String, Object))
			Me.FeedItem = navigationParameter
		End Sub

		Protected async Function SavePinnedFeedItem() As Task
            Dim pinnedFeedItems As List(Of FeedItem) = Await Me.storage.LoadAsync(Of List(Of FeedItem))(Constants.PinnedFeedItemsKey)

			If pinnedFeedItems Is Nothing Then
                pinnedFeedItems = New List(Of FeedItem)
			End If

			pinnedFeedItems.Add(feedItem_Renamed)
			await Me.storage.SaveAsync(Constants.PinnedFeedItemsKey, pinnedFeedItems)
		End Function

		Protected async Function RemovePinnedFeedItem() As Task
            Dim pinnedFeedItems As List(Of FeedItem) = Await Me.storage.LoadAsync(Of List(Of FeedItem))(Constants.PinnedFeedItemsKey)

			If pinnedFeedItems IsNot Nothing Then
                Dim pinnedFeedItem As FeedItem = pinnedFeedItems.FirstOrDefault(Function(fi) fi.Id = Me.FeedItem.Id)
				If pinnedFeedItem IsNot Nothing Then
					pinnedFeedItems.Remove(pinnedFeedItem)
				End If

				await Me.storage.SaveAsync(Constants.PinnedFeedItemsKey, pinnedFeedItems)
			End If
		End Function

'INSTANT VB NOTE: The variable feedItem was renamed since Visual Basic does not allow class members with the same name:
		Private feedItem_Renamed As FeedItem
        Public Property FeedItem As FeedItem
            Get
                Return Me.feedItem_Renamed
            End Get
            Set(ByVal value As FeedItem)
                Me.SetProperty(Me.feedItem_Renamed, value)
            End Set
        End Property

'INSTANT VB NOTE: The variable isFeedItemPinned was renamed since Visual Basic does not allow class members with the same name:
		Private isFeedItemPinned_Renamed As Boolean
        Public Property IsFeedItemPinned As Boolean
            Get
                Return Me.isFeedItemPinned_Renamed
            End Get
            Set(ByVal value As Boolean)
                Me.SetProperty(Me.isFeedItemPinned_Renamed, value)
            End Set
        End Property
	End Class
End Namespace
