﻿Imports Charmed.Sample.Models
Imports System.Text
Imports System.Threading.Tasks
Imports Windows.ApplicationModel.DataTransfer

Namespace Charmed.Sample.ViewModels
	Public NotInheritable Class Win8FeedItemViewModel
		Inherits FeedItemViewModel
		Private ReadOnly shareManager As IShareManager

		Public Sub New(ByVal serializer As ISerializer, ByVal storage As IStorage, ByVal secondaryPinner As ISecondaryPinner, ByVal shareManager As IShareManager)
			MyBase.New(serializer, storage, secondaryPinner)
			Me.shareManager = shareManager
		End Sub

		Public Overrides Sub LoadState(ByVal navigationParameter As Models.FeedItem, ByVal pageState As Dictionary(Of String, Object))
			MyBase.LoadState(navigationParameter, pageState)

			Me.IsFeedItemPinned = Me.secondaryPinner.IsPinned(FormatSecondaryTileId())

			Me.shareManager.Initialize()
			Me.shareManager.OnShareRequested = AddressOf ShareRequested
		End Sub

		Public Overrides Sub SaveState(ByVal pageState As Dictionary(Of String, Object))
			MyBase.SaveState(pageState)

			Me.shareManager.Cleanup()
		End Sub

		Public async Function Pin(ByVal anchorElement As Windows.UI.Xaml.FrameworkElement) As Task
			' Pin the feed item, then save it locally to make sure it is still available
			' when they return.
            Dim tileInfo As TileInfo = New TileInfo(Me.FormatSecondaryTileId, Me.FeedItem.Title, Me.FeedItem.Title, Windows.UI.StartScreen.TileOptions.ShowNameOnLogo Or Windows.UI.StartScreen.TileOptions.ShowNameOnWideLogo, New Uri("ms-appx:///Assets/Logo.png"), New Uri("ms-appx:///Assets/WideLogo.png"), anchorElement, Windows.UI.Popups.Placement.Above, Me.FeedItem.Id.ToString())

			Me.IsFeedItemPinned = await Me.secondaryPinner.Pin(tileInfo)

			If Me.IsFeedItemPinned Then
				await SavePinnedFeedItem()
			End If
		End Function

		Public async Function Unpin(ByVal anchorElement As Windows.UI.Xaml.FrameworkElement) As Task
			' Unpin, then delete the feed item locally.
            Dim tileInfo As TileInfo = New TileInfo(Me.FormatSecondaryTileId, anchorElement, Windows.UI.Popups.Placement.Above)
            Me.IsFeedItemPinned = Not Await Me.secondaryPinner.Unpin(tileInfo)

			If Not Me.IsFeedItemPinned Then
				await RemovePinnedFeedItem()
			End If
		End Function

		Private Sub ShareRequested(ByVal dataPackage As DataPackage)
			' Set as many data types as we can.
			dataPackage.Properties.Title = Me.FeedItem.Title

			' Add a Uri
			dataPackage.SetUri(Me.FeedItem.Link)

			' Add a text only version
            Dim text As String = String.Format("Check this out! {0} ({1})", Me.FeedItem.Title, Me.FeedItem.Link)
			dataPackage.SetText(text)

			' Add an HTML version.
            Dim htmlBuilder As StringBuilder = New StringBuilder()
			htmlBuilder.AppendFormat("<p>Check this out!</p>", Me.FeedItem.Author)
			htmlBuilder.AppendFormat("<p><a href='{0}'>{1}</a></p>", Me.FeedItem.Link, Me.FeedItem.Title)
            Dim html As String = HtmlFormatHelper.CreateHtmlFormat(htmlBuilder.ToString)
			dataPackage.SetHtmlFormat(html)
		End Sub

		Private Function FormatSecondaryTileId() As String
			Return String.Format(Constants.SecondaryIdFormat, Me.FeedItem.Id)
		End Function
	End Class
End Namespace
