﻿Imports Microsoft.Phone.Controls
Imports Microsoft.Phone.Shell
Imports System.Collections
Imports System.Collections.ObjectModel
Imports System.Collections.Specialized
Imports System.Windows
Imports System

Namespace Charmed.ApplicationBar
    ''' <summary>
    ''' Application Bar which allows Menu Items to be databound.
    ''' </summary>
    Public NotInheritable Class BindableApplicationBar
        Inherits FrameworkElement
        Implements IApplicationBar
        Private ReadOnly applicationBar As Microsoft.Phone.Shell.ApplicationBar

        Public Shared ReadOnly MenuItemsProperty As DependencyProperty = DependencyProperty.Register("MenuItems", GetType(IList), GetType(BindableApplicationBar), New PropertyMetadata(New ObservableCollection(Of Object), AddressOf OnBindableMenuItemsChanged))

        Public Shared ReadOnly ButtonsProperty As DependencyProperty = DependencyProperty.Register("Buttons", GetType(IList), GetType(BindableApplicationBar), New PropertyMetadata(New ObservableCollection(Of Object), AddressOf OnBindableButtonsChanged))

        Public Sub New()
            Me.applicationBar = New Microsoft.Phone.Shell.ApplicationBar
            AddHandler Me.applicationBar.StateChanged, AddressOf OnStateChanged

            Me.buttons_Renamed = New ObservableCollection(Of ApplicationBarIconButton)
            AddHandler Me.buttons_Renamed.CollectionChanged, AddressOf OnButtonsCollectionChanged

            AddHandler Me.Loaded, AddressOf OnLoaded
        End Sub

        Private Overloads Property Opacity As Double Implements IApplicationBar.Opacity
            Get
                Return Opacity
            End Get

            Set(value As Double)

            End Set
        End Property

        Public ReadOnly Property MenuItemss As System.Collections.IList Implements IApplicationBar.MenuItems
            Get
                Return Nothing
            End Get

        End Property

        Private Sub OnMenuItemsCollectionChanged(ByVal sender As Object, ByVal e As System.Collections.Specialized.NotifyCollectionChangedEventArgs)
            If e.Action = NotifyCollectionChangedAction.Reset Then
                Me.applicationBar.MenuItems.Clear()
            ElseIf e.Action = NotifyCollectionChangedAction.Add Then
                If e.NewItems IsNot Nothing Then
                    For Each item In e.NewItems
                        Dim menuItem = GetMenuItem(Me, item)
                        Me.applicationBar.MenuItems.Add(menuItem)
                    Next item
                End If
            End If
        End Sub

        Private Sub OnButtonsCollectionChanged(ByVal sender As Object, ByVal e As System.Collections.Specialized.NotifyCollectionChangedEventArgs)
            If e.Action = NotifyCollectionChangedAction.Reset Then
                Me.applicationBar.Buttons.Clear()
            ElseIf e.Action = NotifyCollectionChangedAction.Add Then
                If e.NewItems IsNot Nothing Then
                    For Each item In e.NewItems
                        Dim button = GetButton(Me, item)
                        Me.applicationBar.Buttons.Add(button)
                    Next item
                End If
            End If
        End Sub

        Private Sub OnLoaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
            Dim page = BindableApplicationBar.FindRootPage(Me)
            If page IsNot Nothing Then
                page.ApplicationBar = Me.applicationBar
            End If

            If (Not AreMenuItemsLoaded) AndAlso Me.MenuItems IsNot Nothing Then
                For Each item In Me.MenuItems
                    Dim menuItem = GetMenuItem(Me, item)
                    Me.applicationBar.MenuItems.Add(menuItem)
                Next item
            End If

            If (Not AreButtonsLoaded) AndAlso Me.Buttons IsNot Nothing Then
                For Each button In Me.Buttons
                    Dim bindableButton = TryCast(button, BindableApplicationBarIconButton)
                    If bindableButton IsNot Nothing Then
                        bindableButton.DataContext = Me.DataContext
                    End If
                Next button
            End If
        End Sub

        Private Sub OnStateChanged(ByVal sender As Object, ByVal e As ApplicationBarStateChangedEventArgs)
            RaiseEvent StateChanged(Me, e)
        End Sub

        Private Shared Sub OnBindableMenuItemsChanged(ByVal d As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
            If d Is Nothing Then
                Throw New ArgumentNullException("d")
            End If

            Dim bindableApplicationBar = CType(d, BindableApplicationBar)
            bindableApplicationBar.MenuItems.Clear()
            Dim items = TryCast(e.NewValue, IList)
            If items IsNot Nothing Then
                For Each item In items
                    Dim menuItem = GetMenuItem(bindableApplicationBar, item)
                    bindableApplicationBar.MenuItems.Add(menuItem)
                Next item
            End If

            If e.OldValue IsNot Nothing Then
                Dim oldObservableItems = TryCast(e.NewValue, INotifyCollectionChanged)
                If oldObservableItems IsNot Nothing Then
                    RemoveHandler oldObservableItems.CollectionChanged, AddressOf bindableApplicationBar.OnMenuItemsCollectionChanged
                End If
            End If

            Dim observableItems = TryCast(e.NewValue, INotifyCollectionChanged)
            If observableItems IsNot Nothing Then
                AddHandler observableItems.CollectionChanged, AddressOf bindableApplicationBar.OnMenuItemsCollectionChanged
            End If
            bindableApplicationBar.AreMenuItemsLoaded = True
        End Sub

        Private Shared Sub OnBindableButtonsChanged(ByVal d As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
            If d Is Nothing Then
                Throw New ArgumentNullException("d")
            End If

            Dim bindableApplicationBar = CType(d, BindableApplicationBar)
            bindableApplicationBar.Buttons.Clear()
            Dim items = TryCast(e.NewValue, IList)
            If items IsNot Nothing Then
                For Each item In items
                    Dim menuItem = GetButton(bindableApplicationBar, item)
                    bindableApplicationBar.Buttons.Add(menuItem)
                Next item
            End If

            If e.OldValue IsNot Nothing Then
                Dim oldObservableItems = TryCast(e.NewValue, INotifyCollectionChanged)
                If oldObservableItems IsNot Nothing Then
                    RemoveHandler oldObservableItems.CollectionChanged, AddressOf bindableApplicationBar.OnButtonsCollectionChanged
                End If
            End If

            Dim observableItems = TryCast(e.NewValue, INotifyCollectionChanged)
            If observableItems IsNot Nothing Then
                AddHandler observableItems.CollectionChanged, AddressOf bindableApplicationBar.OnButtonsCollectionChanged
            End If
            bindableApplicationBar.AreMenuItemsLoaded = True
        End Sub

        Private Shared Function GetMenuItem(ByVal bindableApplicationBar As BindableApplicationBar, ByVal item As Object) As Object
            Dim bindableMenuItem = TryCast(item, BindableApplicationBarMenuItem)
            If bindableMenuItem IsNot Nothing Then
                bindableMenuItem.DataContext = bindableApplicationBar.DataContext
                bindableMenuItem.DataItem = item
                If Not String.IsNullOrEmpty(bindableApplicationBar.MenuItemClickMethodName) Then
                    bindableMenuItem.ClickMethodName = bindableApplicationBar.MenuItemClickMethodName
                End If
                Return bindableMenuItem
            ElseIf TypeOf item Is ApplicationBarMenuItem Then
                Return item
            Else
                Dim textProperty = item.GetType.GetProperty(bindableApplicationBar.TextMemberPath)
                Dim menuItem = New BindableApplicationBarMenuItem
                menuItem.Text = textProperty.GetValue(item).ToString
                menuItem.DataContext = bindableApplicationBar.DataContext
                menuItem.ClickMethodName = bindableApplicationBar.MenuItemClickMethodName
                menuItem.DataItem = item
                Return menuItem
            End If
        End Function

        Private Shared Function GetButton(ByVal bindableApplicationBar As BindableApplicationBar, ByVal item As Object) As Object
            Dim bindableButton = TryCast(item, BindableApplicationBarIconButton)
            If bindableButton IsNot Nothing Then
                bindableButton.DataContext = bindableApplicationBar.DataContext
                bindableButton.DataItem = item
                If Not String.IsNullOrEmpty(bindableApplicationBar.MenuItemClickMethodName) Then
                    bindableButton.ClickMethodName = bindableApplicationBar.MenuItemClickMethodName
                End If
                Return bindableButton
            ElseIf TypeOf item Is ApplicationBarMenuItem Then
                Return item
            Else
                Dim textProperty = item.GetType.GetProperty(bindableApplicationBar.TextMemberPath)
                Dim menuItem = New BindableApplicationBarMenuItem
                menuItem.Text = textProperty.GetValue(item).ToString
                menuItem.DataContext = bindableApplicationBar.DataContext
                menuItem.ClickMethodName = bindableApplicationBar.MenuItemClickMethodName
                menuItem.DataItem = item
                Return menuItem
            End If
        End Function

        Private Shared Function FindRootPage(ByVal control As FrameworkElement) As PhoneApplicationPage
            If control Is Nothing Then
                Throw New ArgumentNullException("control")
            End If

            If control IsNot Nothing AndAlso Not (TypeOf control Is PhoneApplicationPage) Then
                control = FindRootPage(TryCast(control.Parent, FrameworkElement))
            End If
            Return TryCast(control, PhoneApplicationPage)
        End Function

        Public Property BackgroundColor As System.Windows.Media.Color Implements IApplicationBar.BackgroundColor
            Get
                Return Me.applicationBar.BackgroundColor
            End Get
            Set(ByVal value As System.Windows.Media.Color)
                Me.applicationBar.BackgroundColor = value
            End Set
        End Property

        Private buttons_Renamed As ObservableCollection(Of ApplicationBarIconButton)
        Public ReadOnly Property Buttons As System.Collections.IList Implements IApplicationBar.Buttons
            Get
                Return buttons_Renamed
            End Get
        End Property

        Public ReadOnly Property DefaultSize As Double Implements IApplicationBar.DefaultSize
            Get
                Return Me.applicationBar.DefaultSize
            End Get
        End Property

        Public Property ForegroundColor As System.Windows.Media.Color Implements IApplicationBar.ForegroundColor
            Get
                Return Me.applicationBar.ForegroundColor
            End Get
            Set(ByVal value As System.Windows.Media.Color)
                Me.applicationBar.ForegroundColor = value
            End Set
        End Property

        Public Property IsMenuEnabled As Boolean Implements IApplicationBar.IsMenuEnabled
            Get
                Return Me.applicationBar.IsMenuEnabled
            End Get
            Set(ByVal value As Boolean)
                Me.applicationBar.IsMenuEnabled = value
            End Set
        End Property

        Public Property IsVisible As Boolean Implements IApplicationBar.IsVisible
            Get
                Return Me.applicationBar.IsVisible
            End Get
            Set(ByVal value As Boolean)
                Me.applicationBar.IsVisible = value
            End Set
        End Property

        <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly")>
        Public Property MenuItems As System.Collections.IList
            Get
                Return CType(Me.GetValue(BindableApplicationBar.MenuItemsProperty), IList)
            End Get
            Set(ByVal value As System.Collections.IList)
                Me.SetValue(BindableApplicationBar.MenuItemsProperty, value)
            End Set
        End Property

        Public ReadOnly Property MiniSize As Double Implements IApplicationBar.MiniSize
            Get
                Return Me.applicationBar.MiniSize
            End Get
        End Property

        Public Property Mode As ApplicationBarMode Implements IApplicationBar.Mode
            Get
                Return Me.applicationBar.Mode
            End Get
            Set(ByVal value As ApplicationBarMode)
                Me.applicationBar.Mode = value
            End Set
        End Property

        Public Property TextMemberPath As String
        Public Property MenuItemClickMethodName As String

        Private Property AreMenuItemsLoaded As Boolean
        Private Property AreButtonsLoaded As Boolean

        Public Event StateChanged As EventHandler(Of ApplicationBarStateChangedEventArgs) Implements IApplicationBar.StateChanged
    End Class
End Namespace
