﻿Imports Charmed.Converters
Imports Microsoft.VisualStudio.TestPlatform.UnitTestFramework
Imports System

Namespace Charmed.Tests.Converters
    <TestClass>
    Public Class DateTimeToStringConverterTests
        <TestMethod>
        Public Sub Convert()
            ' Arrange
            Dim converter As DateTimeToStringConverter = New DateTimeToStringConverter()

            Dim [date] As Date = Date.Now

            ' Act
            Dim result As String = CStr(converter.Convert([date], Nothing, Nothing, Nothing))

            ' Assert
            Assert.AreEqual([date].ToString("d"), result)
        End Sub

        <TestMethod>
        Public Sub Convert_CustomFormat()
            ' Arrange
            Dim converter As DateTimeToStringConverter = New DateTimeToStringConverter()
            converter.Format = "dd"

            Dim [date] As Date = Date.Now

            ' Act
            Dim result As String = CStr(converter.Convert([date], Nothing, Nothing, Nothing))

            ' Assert
            Assert.AreEqual([date].ToString("dd"), result)
        End Sub

        <TestMethod>
        Public Sub Convert_Null()
            ' Arrange
            Dim converter As DateTimeToStringConverter = New DateTimeToStringConverter()

            ' Act
            Dim result As String = CStr(converter.Convert(Nothing, Nothing, Nothing, Nothing))

            ' Assert
            Assert.AreEqual(String.Empty, result)
        End Sub

        <TestMethod>
        Public Sub Convert_NotDateTime()
            ' Arrange
            Dim converter As DateTimeToStringConverter = New DateTimeToStringConverter()

            ' Act
            Dim result As String = CStr(converter.Convert(New Object(), Nothing, Nothing, Nothing))

            ' Assert
            Assert.AreEqual(String.Empty, result)
        End Sub

        <TestMethod>
        Public Sub ConvertBack()
            ' Arrange
            Dim converter As DateTimeToStringConverter = New DateTimeToStringConverter()

            Dim [date] As Date = Date.Today

            ' Act
            Dim result As Date = CDate(converter.ConvertBack([date].ToString("d"), Nothing, Nothing, Nothing))

            ' Assert
            Assert.AreEqual([date], result)
        End Sub

        <TestMethod>
        Public Sub ConvertBack_Null_NotNullable()
            ' Arrange
            Dim converter As DateTimeToStringConverter = New DateTimeToStringConverter()

            ' Act
            Dim result As Date = CDate(converter.ConvertBack(Nothing, Nothing, Nothing, Nothing))

            ' Assert
            Assert.AreEqual(New Date(), result)
        End Sub

        <TestMethod>
        Public Sub ConvertBack_Null_Nullable()
            ' Arrange
            Dim converter As DateTimeToStringConverter = New DateTimeToStringConverter()
            converter.IsDateTimeNullable = True

            ' Act
            Dim result As Object = converter.ConvertBack(Nothing, Nothing, Nothing, Nothing)

            ' Assert
            Assert.IsNull(result)
        End Sub

        <TestMethod>
        Public Sub ConvertBack_NotParsable_NotNullable()
            ' Arrange
            Dim converter As DateTimeToStringConverter = New DateTimeToStringConverter()

            ' Act
            Dim result As Date = CDate(converter.ConvertBack(Guid.NewGuid().ToString, Nothing, Nothing, Nothing))

            ' Assert
            Assert.AreEqual(New Date(), result)
        End Sub

        <TestMethod>
        Public Sub ConvertBack_NotParsable_Nullable()
            ' Arrange
            Dim converter As DateTimeToStringConverter = New DateTimeToStringConverter()
            converter.IsDateTimeNullable = True

            ' Act
            Dim result As Object = converter.ConvertBack(Guid.NewGuid().ToString, Nothing, Nothing, Nothing)

            ' Assert
            Assert.IsNull(result)
        End Sub
    End Class
End Namespace
