﻿Imports Charmed.Helpers
Imports Microsoft.VisualStudio.TestPlatform.UnitTestFramework
Imports Microsoft.VisualStudio.TestPlatform.UnitTestFramework.AppContainer
Imports Windows.UI.Xaml.Automation.Peers
Imports Windows.UI.Xaml.Automation.Provider
Imports Windows.UI.Xaml.Controls
Imports System

Namespace Charmed.Tests.Helpers
    <TestClass>
    Public Class ButtonHelperTests
        <UITestMethod>
        Public Sub Click()
            ' Arrange
            Dim mock As Mock = New Mock()
            Dim button As Button = New Button()
            button.DataContext = mock

            ButtonHelper.SetMethodName(button, "DoStuff")

            ' Act
            Dim peer As ButtonAutomationPeer = New ButtonAutomationPeer(button)
            Dim invokeProvider As IInvokeProvider = TryCast(peer.GetPattern(PatternInterface.Invoke), IInvokeProvider)
            invokeProvider.Invoke()

            ' Assert
            Microsoft.VisualStudio.TestPlatform.UnitTestFramework.Assert.IsTrue(mock.WasButtonClicked)
        End Sub

        <UITestMethod>
        Public Sub Click_WithParameter()
            ' Arrange
            Dim mock As Mock = New Mock()
            Dim button As Button = New Button()
            button.DataContext = mock

            Dim expectedParameter As String = Guid.NewGuid().ToString

            ButtonHelper.SetMethodName(button, "DoStuffWithParameter")
            ButtonHelper.SetParameter(button, expectedParameter)

            ' Act
            Dim peer As ButtonAutomationPeer = New ButtonAutomationPeer(button)
            Dim invokeProvider As IInvokeProvider = TryCast(peer.GetPattern(PatternInterface.Invoke), IInvokeProvider)
            invokeProvider.Invoke()

            ' Assert
            Microsoft.VisualStudio.TestPlatform.UnitTestFramework.Assert.AreEqual(expectedParameter, mock.Parameter, "Parameter")
        End Sub

        <UITestMethod>
        Public Sub Click_WithButtonParameter()
            ' Arrange
            Dim mock As Mock = New Mock()
            Dim button As Button = New Button()
            button.DataContext = mock

            ButtonHelper.SetMethodName(button, "DoStuffWithButtonParameter")

            ' Act
            Dim peer As ButtonAutomationPeer = New ButtonAutomationPeer(button)
            Dim invokeProvider As IInvokeProvider = TryCast(peer.GetPattern(PatternInterface.Invoke), IInvokeProvider)
            invokeProvider.Invoke()

            ' Assert
            Microsoft.VisualStudio.TestPlatform.UnitTestFramework.Assert.AreSame(button, mock.ButtonParameter, "Parameter")
        End Sub

        <UITestMethod>
        Public Sub Click_WithButtonAndStringParameter()
            ' Arrange
            Dim mock As Mock = New Mock()
            Dim button As Button = New Button()
            button.DataContext = mock

            Dim expectedParameter As String = Guid.NewGuid().ToString

            ButtonHelper.SetMethodName(button, "DoStuffWithButtonAndStringParameter")
            ButtonHelper.SetParameter(button, expectedParameter)

            ' Act
            Dim peer As ButtonAutomationPeer = New ButtonAutomationPeer(button)
            Dim invokeProvider As IInvokeProvider = TryCast(peer.GetPattern(PatternInterface.Invoke), IInvokeProvider)
            invokeProvider.Invoke()

            ' Assert
            Microsoft.VisualStudio.TestPlatform.UnitTestFramework.Assert.AreSame(button, mock.ButtonParameter, "Button Parameter")
            Microsoft.VisualStudio.TestPlatform.UnitTestFramework.Assert.AreEqual(expectedParameter, mock.Parameter, "Parameter")
        End Sub

        <UITestMethod>
        Public Sub Click_SetContext_WithButtonAndStringParameter()
            ' Arrange
            Dim mock As Mock = New Mock()
            Dim button As Button = New Button()

            Dim expectedParameter As String = Guid.NewGuid().ToString

            ButtonHelper.SetMethodName(button, "DoStuffWithButtonAndStringParameter")
            ButtonHelper.SetParameter(button, expectedParameter)
            ButtonHelper.SetMethodContext(button, mock)

            ' Act
            Dim peer As ButtonAutomationPeer = New ButtonAutomationPeer(button)
            Dim invokeProvider As IInvokeProvider = TryCast(peer.GetPattern(PatternInterface.Invoke), IInvokeProvider)
            invokeProvider.Invoke()

            ' Assert
            Microsoft.VisualStudio.TestPlatform.UnitTestFramework.Assert.AreSame(button, mock.ButtonParameter, "Button Parameter")
            Microsoft.VisualStudio.TestPlatform.UnitTestFramework.Assert.AreEqual(expectedParameter, mock.Parameter, "Parameter")
        End Sub

        Public Class Mock
            Public Property WasButtonClicked As Boolean
            Public Sub DoStuff()
                Me.WasButtonClicked = True
            End Sub

            Public Property Parameter As String
            Public Sub DoStuffWithParameter(ByVal parameter As String)
                Me.Parameter = parameter
            End Sub

            Public Property ButtonParameter As Button
            Public Sub DoStuffWithButtonParameter(ByVal buttonParameter As Button)
                Me.ButtonParameter = buttonParameter
            End Sub

            Public Sub DoStuffWithButtonAndStringParameter(ByVal buttonParameter As Button, ByVal parameter As String)
                Me.Parameter = parameter
                Me.ButtonParameter = buttonParameter
            End Sub
        End Class
    End Class
End Namespace
