﻿Imports Charmed.Messaging
Imports Microsoft.VisualStudio.TestPlatform.UnitTestFramework
Imports System.Collections.Generic
Imports System

Namespace Charmed.Tests.Messaging
    <TestClass>
    Public Class MessageBusTests
        Private Property Messages As List(Of Object)
        Private Sub Handler(ByVal message As Object)
            Messages.Add(message)
        End Sub

        <TestInitialize>
        Public Sub TestInitialize()
            Messages = New List(Of Object)()
        End Sub

        <TestMethod>
        Public Sub SubscribeNotExisting()
            ' Arrange
            Dim bus As MessageBus = New MessageBus()
            bus.Subscribe(Of Object)(AddressOf Handler)

            ' Act
            Dim message As Object = New Object()
            bus.Publish(Of Object)(message)

            ' Assert
            Assert.AreEqual(1, Messages.Count)
            Assert.AreSame(message, Messages(0))
        End Sub

        <TestMethod>
        Public Sub SubscribeExisting()
            ' Assert
            Dim bus As MessageBus = New MessageBus()
            bus.Subscribe(Of Object)(AddressOf Handler)
            bus.Subscribe(Of Object)(AddressOf Handler)

            ' Act
            Dim message As Object = New Object()
            bus.Publish(Of Object)(message)

            ' Assert
            Assert.AreEqual(2, Messages.Count)
            Assert.AreSame(message, Messages(0))
            Assert.AreSame(message, Messages(1))
        End Sub

        <TestMethod>
        Public Sub Unsubscribe()
            ' Arrange
            Dim bus As MessageBus = New MessageBus()
            bus.Subscribe(Of Object)(AddressOf Handler)

            ' Act
            bus.Unsubscribe(Of Object)(AddressOf Handler)

            Dim message As Object = New Object()
            bus.Publish(Of Object)(message)

            ' Assert
            Assert.AreEqual(0, Messages.Count)
        End Sub

        Private Class TestSubscriber
            Public Property Messages As List(Of Object)

            Public Sub New()
                Messages = New List(Of Object)
            End Sub

            Public Sub Handler(ByVal message As Object)
                Messages.Add(message)
            End Sub
        End Class

        <TestMethod>
        Public Sub Unsubscribe_MultipleInstances_SameClass()
            ' Arrange
            Dim bus As MessageBus = New MessageBus()

            ' Subscribe two objects of the same type.
            Dim subscriber1 As TestSubscriber = New TestSubscriber()
            bus.Subscribe(Of Object)(AddressOf subscriber1.Handler)

            Dim subscriber2 As TestSubscriber = New TestSubscriber()
            bus.Subscribe(Of Object)(AddressOf subscriber2.Handler)

            ' Act

            ' Unsubscribe the second of the objects and make sure the other first one still gets messages.
            bus.Unsubscribe(Of Object)(AddressOf subscriber2.Handler)

            Dim message As Object = New Object()
            bus.Publish(Of Object)(message)

            ' Assert
            Assert.AreEqual(0, subscriber2.Messages.Count)
            Assert.AreEqual(1, subscriber1.Messages.Count)
        End Sub

        <TestMethod>
        Public Sub Publish()
            ' Arrange
            Dim bus As MessageBus = New MessageBus()
            bus.Subscribe(Of Object)(AddressOf Handler)

            ' Act
            Dim message As String = Guid.NewGuid().ToString
            bus.Publish(Of Object)(message)

            ' Assert
            Assert.AreEqual(1, Messages.Count)
        End Sub

        <TestMethod>
        Public Sub PublishDifferent()
            ' Arrange
            Dim bus As MessageBus = New MessageBus()
            bus.Subscribe(Of Object)(AddressOf Handler)

            ' Act
            Dim message As String = Guid.NewGuid().ToString
            bus.Publish(Of String)(message)

            ' Assert
            Assert.AreEqual(0, Messages.Count)
        End Sub

        <TestMethod>
        Public Sub PublishAndUnsubscribe()
            ' Arrange
            Dim bus As MessageBus = New MessageBus()
            Dim handler As Action(Of Object) = Nothing
            handler = Sub(m)
                          Messages.Add(m)

                          ' Unsubscribe during the publish handler.
                          bus.Unsubscribe(Of Object)(handler)
                      End Sub
            bus.Subscribe(Of Object)(handler)
            bus.Subscribe(Of Object)(Sub(m) Messages.Add(m))

            ' Act
            Dim message As String = Guid.NewGuid().ToString
            bus.Publish(Of Object)(message)
            bus.Publish(Of Object)(message)

            ' Assert
            Assert.AreEqual(3, Messages.Count)
        End Sub
    End Class
End Namespace
