﻿Imports System.Linq
Imports System.Reflection
Imports System.Text
Imports System.Threading.Tasks
Imports System
Imports System.Collections.Generic

Namespace Charmed.Container
    Public Class DuplicateRegistrationException
        Inherits Exception
       
        Public Sub New(ByVal message As String)
            MyBase.New(message)
        End Sub
        Public Sub New(ByVal message As String, ByVal inner As Exception)
            MyBase.New(message, inner)
        End Sub
    End Class

    Public NotInheritable Class SimpleContainer
        Implements IContainer
        Private Property Types As Dictionary(Of Type, Type)
        Private Property Instances As Dictionary(Of Type, Object)

        Public Sub New()
            Types = New Dictionary(Of Type, Type)
            Instances = New Dictionary(Of Type, Object)
        End Sub

        Public Sub Register(Of TClass)() Implements IContainer.Register
            If IsAlreadyRegistered(Of TClass)() Then
                Throw New DuplicateRegistrationException(String.Format("Service of Type '{0}' is already registered.", GetType(TClass).Name))
            End If

            Types.Add(GetType(TClass), GetType(TClass))
        End Sub

        Public Sub Register(Of TService, TClass As TService)() Implements IContainer.Register
            If IsAlreadyRegistered(Of TService)() Then
                Throw New DuplicateRegistrationException(String.Format("Service of Type '{0}' is already registered.", GetType(TService).Name))
            End If

            Types.Add(GetType(TService), GetType(TClass))
        End Sub

        Public Sub RegisterInstance(Of TService)(ByVal instance As TService) Implements IContainer.RegisterInstance
            If IsAlreadyRegistered(Of TService)() Then
                Throw New DuplicateRegistrationException(String.Format("Service of Type '{0}' is already registered.", GetType(TService).Name))
            End If

            Instances.Add(GetType(TService), instance)
        End Sub

        Public Function Resolve(Of TService)() As TService Implements IContainer.Resolve
            Return CType(Resolve(GetType(TService)), TService)
        End Function

        Public Function Resolve(ByVal type As Type) As Object Implements IContainer.Resolve
            If Not Types.ContainsKey(type) Then
                If Not Instances.ContainsKey(type) Then
                    Throw New NotSupportedException(String.Format("No registration found for service of Type '{0}'.", type.Name))
                Else
                    Return Instances(type)
                End If
            Else
                Dim createdType As Type = Types(type)

                Dim constructors As IEnumerable(Of System.Reflection.ConstructorInfo) = createdType.GetTypeInfo.DeclaredConstructors
                Dim mostSpecificConstructor As ConstructorInfo = Nothing
                Dim constructor As System.Reflection.ConstructorInfo
                For Each constructor In constructors
                    If mostSpecificConstructor Is Nothing OrElse mostSpecificConstructor.GetParameters.Length < constructor.GetParameters.Length Then
                        mostSpecificConstructor = constructor
                    End If

                Next constructor

                Dim constructorParameters As List(Of Object) = New List(Of Object)
                For Each param As ParameterInfo In mostSpecificConstructor.GetParameters
                    constructorParameters.Add(Resolve(param.ParameterType))
                Next param

                Return Activator.CreateInstance(createdType, constructorParameters.ToArray)
            End If
        End Function

        Private Function IsAlreadyRegistered(Of T)() As Boolean
            Return Instances.ContainsKey(GetType(T)) OrElse Types.ContainsKey(GetType(T))
        End Function
    End Class
End Namespace
