﻿Namespace Charmed
	''' <summary>
	''' Interface for saving application settings and data.
	''' </summary>
	Public Interface ISettings
		''' <summary>
		''' Adds or updates setting data.
		''' </summary>
		''' <param name="key">The key for the data.</param>
		''' <param name="value">The data.</param>
		Sub AddOrUpdate(ByVal key As String, ByVal value As Object)

		''' <summary>
		''' Tries to get setting data.
		''' </summary>
		''' <typeparam name="T">The type of the result.</typeparam>
		''' <param name="key">The key for the data.</param>
		''' <param name="value">The output data value.</param>
		''' <returns>Returns true if the key existed and the value was assigned to value.</returns>
        Function TryGetValue(Of T)(ByVal key As String, <System.Runtime.InteropServices.Out> ByRef value As T) As Boolean

		''' <summary>
		''' Removes setting data.
		''' </summary>
		''' <param name="key">The key for the data.</param>
		''' <returns>Returns true if the data value existed and was removed and false if it did not exist.</returns>
		Function Remove(ByVal key As String) As Boolean

		''' <summary>
		''' Checks whether there is a setting for the given key.
		''' </summary>
		''' <param name="key">The key to check for.</param>
		''' <returns>Returns true if there is a setting for the given key and false if not.</returns>
		Function ContainsKey(ByVal key As String) As Boolean
	End Interface
End Namespace
