﻿Imports System

Namespace Charmed.Messaging
    ''' <summary>
    ''' Interface for a Message Bus.  A Message Bus is a central messaging system for
    ''' an application.
    ''' </summary>
    Public Interface IMessageBus
        ''' <summary>
        ''' Subscribes an action to a particular message type.  When that message type
        ''' is published, this action will be called.
        ''' </summary>
        ''' <typeparam name="TMessage">The type of message to listen for.</typeparam>
        ''' <param name="handler">
        ''' The action which will be called when a message of type <typeparamref name="TMessage"/>
        ''' is published.
        ''' </param>
        Sub Subscribe(Of TMessage)(ByVal handler As Action(Of TMessage))

        ''' <summary>
        ''' Unsubscribes an action from a particular message type.
        ''' </summary>
        ''' <typeparam name="TMessage">The type of message to stop listening for.</typeparam>
        ''' <param name="handler">
        ''' The action which is to be unsubscribed from messages of type <typeparamref name="TMessage"/>.
        ''' </param>
        Sub Unsubscribe(Of TMessage)(ByVal handler As Action(Of TMessage))

        ''' <summary>
        ''' Publishes a message to any subscribers of a particular message type.
        ''' </summary>
        ''' <typeparam name="TMessage">The type of message to publish.</typeparam>
        ''' <param name="message">The message to be published</param>
        Sub Publish(Of TMessage)(ByVal message As TMessage)
    End Interface
End Namespace
