﻿Imports Charmed.Container
Imports System.Reflection
Imports System
Imports System.Text.RegularExpressions

#If NETFX_CORE Then
Imports Windows.UI.Xaml
Imports Windows.UI.Xaml.Controls
#End If ' NETFX_CORE

Namespace Charmed
    Public NotInheritable Class Navigator
        Implements INavigator
        Private ReadOnly serializer As ISerializer
        Private ReadOnly container As IContainer

#If WINDOWS_PHONE Then
        Private ReadOnly frame As Microsoft.Phone.Controls.PhoneApplicationFrame
#End If ' WINDOWS_PHONE

#If WINDOWS_PHONE Then
        Public Sub New(ByVal serializer As ISerializer, ByVal container As IContainer, ByVal frame As Microsoft.Phone.Controls.PhoneApplicationFrame)
#Else
        Public Sub New(ByVal serializer As ISerializer, ByVal container As IContainer)
#End If ' WINDOWS_PHONE
            Me.serializer = serializer
            Me.container = container

#If WINDOWS_PHONE Then
            Me.frame = frame
#End If ' WINDOWS_PHONE
        End Sub

        Public Sub NavigateToViewModel(Of TViewModel)(Optional ByVal parameter As Object = Nothing) Implements INavigator.NavigateToViewModel
            Dim viewType As Type = ResolveViewType(Of TViewModel)()

#If NETFX_CORE Then
            Dim frame As Frame = CType(Window.Current.Content, Frame)
#End If ' NETFX_CORE

            If parameter IsNot Nothing Then
#If WINDOWS_PHONE Then
                Me.frame.Navigate(ResolveViewUri(viewType, parameter))
#Else
                frame.Navigate(viewType, Me.serializer.Serialize(parameter))
#End If ' WINDOWS_PHONE
            Else
#If WINDOWS_PHONE Then
                Me.frame.Navigate(ResolveViewUri(viewType))
#Else
                frame.Navigate(viewType)
#End If ' WINDOWS_PHONE
            End If
        End Sub

        Public Sub GoBack() Implements INavigator.GoBack
#If WINDOWS_PHONE Then
            Me.frame.GoBack()
#Else
            CType(Window.Current.Content, Frame).GoBack()
#End If ' WINDOWS_PHONE
        End Sub

        Public ReadOnly Property CanGoBack As Boolean Implements INavigator.CanGoBack
            Get
#If WINDOWS_PHONE Then
                Return Me.frame.CanGoBack
#Else
                Return (CType(Window.Current.Content, Frame)).CanGoBack
#End If ' WINDOWS_PHONE
            End Get
        End Property

        Private Shared Function ResolveViewType(Of TViewModel)() As Type
            Dim viewModelType As Type = GetType(TViewModel)

            Dim viewName As String = viewModelType.AssemblyQualifiedName.Replace(viewModelType.Name, viewModelType.Name.Replace("ViewModel", "Page"))

            Return Type.GetType(viewName.Replace("Model", String.Empty))
        End Function

        Private Function ResolveViewUri(ByVal viewType As Type, Optional ByVal parameter As Object = Nothing) As Uri
            Dim queryString As String = String.Empty
            If parameter IsNot Nothing Then
                Dim serializedParameter As String = Me.serializer.Serialize(parameter)
                queryString = String.Format("?parameter={0}", serializedParameter)
            End If

            Dim match As Match = System.Text.RegularExpressions.Regex.Match(viewType.FullName, "\.Views.*")
            If match Is Nothing OrElse match.Captures.Count = 0 Then
                Throw New ArgumentException("Views must exist in Views namespace.")
            End If
            Dim path As String = match.Captures(0).Value.Replace("."c, "/"c)

            Return New Uri(String.Format("{0}.xaml{1}", path, queryString), UriKind.Relative)
        End Function

#If WINDOWS_PHONE Then
        Public Sub RemoveBackEntry() Implements INavigator.RemoveBackEntry
            Me.frame.RemoveBackEntry()
        End Sub
#End If ' WINDOWS_PHONE
    End Class
End Namespace
