﻿Imports System
Imports Windows.UI.Popups

Namespace Charmed
	Public NotInheritable Class TileInfo
#If NETFX_CORE Then
		''' <summary>
		''' Constructor containing info for pinning a tile.
		''' </summary>
		''' <param name="tileId">The Id of the tile to pin.</param>
		''' <param name="shortName">The short name for the tile.</param>
		''' <param name="displayName">The display name for the tile.</param>
		''' <param name="tileOptions">The TileOptions for the tile.</param>
		''' <param name="logoUri">The Uri to the tile logo.</param>
		''' <param name="anchorElement">The anchor element that the pin request dialog will display next to.</param>
		''' <param name="requestPlacement">The Placement value that tells where the pin request dialog displays in relation to anchorElement.</param>
		''' <param name="arguments">Optional arguments to provide for when the tile is activated.</param>
		Public Sub New(ByVal tileId As String, ByVal shortName As String, ByVal displayName As String, ByVal tileOptions As Windows.UI.StartScreen.TileOptions, ByVal logoUri As Uri, ByVal anchorElement As Windows.UI.Xaml.FrameworkElement, ByVal requestPlacement As Placement, Optional ByVal arguments As String = Nothing)
			Me.TileId = tileId
			Me.ShortName = shortName
			Me.DisplayName = displayName
			Me.Arguments = arguments
			Me.TileOptions = tileOptions
			Me.LogoUri = logoUri

			Me.AnchorElement = anchorElement
			Me.RequestPlacement = requestPlacement

			Me.Arguments = arguments
		End Sub

		''' <summary>
		''' Constructor containing info for pinning a tile.
		''' </summary>
		''' <param name="tileId">The Id of the tile to pin.</param>
		''' <param name="shortName">The short name for the tile.</param>
		''' <param name="displayName">The display name for the tile.</param>
		''' <param name="tileOptions">The TileOptions for the tile.</param>
		''' <param name="logoUri">The Uri to the tile logo.</param>
		''' <param name="wideLogoUri">The Uri to the wide tile logo.</param>
		''' <param name="anchorElement">The anchor element that the pin request dialog will display next to.</param>
		''' <param name="requestPlacement">The Placement value that tells where the pin request dialog displays in relation to anchorElement.</param>
		''' <param name="arguments">Optional arguments to provide for when the tile is activated.</param>
		Public Sub New(ByVal tileId As String, ByVal shortName As String, ByVal displayName As String, ByVal tileOptions As Windows.UI.StartScreen.TileOptions, ByVal logoUri As Uri, ByVal wideLogoUri As Uri, ByVal anchorElement As Windows.UI.Xaml.FrameworkElement, ByVal requestPlacement As Placement, Optional ByVal arguments As String = Nothing)
			Me.TileId = tileId
			Me.ShortName = shortName
			Me.DisplayName = displayName
			Me.Arguments = arguments
			Me.TileOptions = tileOptions
			Me.LogoUri = logoUri
			Me.WideLogoUri = wideLogoUri

			Me.AnchorElement = anchorElement
			Me.RequestPlacement = requestPlacement

			Me.Arguments = arguments
		End Sub

		''' <summary>
		''' Constructor containing info for unpinning a tile.
		''' </summary>
		''' <param name="tileId">The Id of the tile to pin.</param>
		''' <param name="anchorElement">The anchor element that the pin request dialog will display next to.</param>
		''' <param name="requestPlacement">The Placement value that tells where the pin request dialog displays in relation to anchorElement.</param>
		Public Sub New(ByVal tileId As String, ByVal anchorElement As Windows.UI.Xaml.FrameworkElement, ByVal requestPlacement As Placement)
			Me.TileId = tileId

			Me.AnchorElement = anchorElement
			Me.RequestPlacement = requestPlacement
		End Sub
#Else
		''' <summary>
		''' Constructor containing info for pinning a tile.
		''' </summary>
		''' <param name="tileId">The Id of the tile to pin.</param>
		''' <param name="displayName">The display name for the tile.</param>
		''' <param name="appName">The name of the app for the tile.</param>
		''' <param name="logoUri">The Uri to the tile logo.</param>
		''' <param name="count">Optional count to display on the tile.</param>
		Public Sub New(ByVal tileId As String, ByVal displayName As String, ByVal appName As String, ByVal logoUri As Uri, Optional ByVal count? As Integer = Nothing)
			Me.TileId = tileId
			Me.DisplayName = displayName
			Me.AppName = appName
			Me.LogoUri = logoUri

			Me.Count = count
		End Sub

		''' <summary>
		''' Constructor containing info for unpinning a tile.
		''' </summary>
		''' <param name="tileId">The Id of the tile to pin.</param>
		Public Sub New(ByVal tileId As String)
			Me.TileId = tileId
		End Sub
#End If ' NETFX_CORE

        Public Property TileId As String
        Public Property ShortName As String
        Public Property DisplayName As String
        Public Property Arguments As String
        Public Property LogoUri As Uri
        Public Property WideLogoUri As Uri

        Public Property AppName As String
        Public Property Count As Integer?

#If NETFX_CORE Then
        Public Property TileOptions As Windows.UI.StartScreen.TileOptions
        Public Property AnchorElement As Windows.UI.Xaml.FrameworkElement
        Public Property RequestPlacement As Placement
#End If ' NETFX_CORE
	End Class
End Namespace
