﻿Imports System.Threading.Tasks
Imports Windows.Foundation
Imports Windows.UI.StartScreen
Imports Windows.UI.Xaml
Imports Windows.UI.Xaml.Media
Imports System

Namespace Charmed
    Public NotInheritable Class Win8SecondaryPinner
        Implements ISecondaryPinner
        Public Async Function Pin(ByVal tileInfo As TileInfo) As Task(Of Boolean) Implements ISecondaryPinner.Pin
            If tileInfo Is Nothing Then
                Throw New ArgumentNullException("tileInfo")
            End If

            Dim isPinned As Boolean = False

            If Not SecondaryTile.Exists(tileInfo.TileId) Then
                Dim secondaryTile As SecondaryTile = New SecondaryTile(tileInfo.TileId, tileInfo.ShortName, tileInfo.DisplayName, tileInfo.Arguments, tileInfo.TileOptions, tileInfo.LogoUri)

                If tileInfo.WideLogoUri IsNot Nothing Then
                    secondaryTile.WideLogo = tileInfo.WideLogoUri
                End If

                isPinned = Await secondaryTile.RequestCreateForSelectionAsync(GetElementRect(tileInfo.AnchorElement), tileInfo.RequestPlacement)
            End If

            Return isPinned
        End Function

        Public Async Function Unpin(ByVal tileInfo As TileInfo) As Task(Of Boolean) Implements ISecondaryPinner.Unpin
            Dim wasUnpinned As Boolean = False

            If SecondaryTile.Exists(tileInfo.TileId) Then
                Dim secondaryTile As SecondaryTile = New SecondaryTile(tileInfo.TileId)
                wasUnpinned = Await secondaryTile.RequestDeleteForSelectionAsync(GetElementRect(tileInfo.AnchorElement), tileInfo.RequestPlacement)
            End If

            Return wasUnpinned
        End Function

        Public Function IsPinned(ByVal tileId As String) As Boolean Implements ISecondaryPinner.IsPinned
            Return SecondaryTile.Exists(tileId)
        End Function

        Private Shared Function GetElementRect(ByVal element As FrameworkElement) As Rect
            Dim buttonTransform As GeneralTransform = element.TransformToVisual(Nothing)
            Dim point As Point = buttonTransform.TransformPoint(New Point())
            Return New Rect(point, New Size(element.ActualWidth, element.ActualHeight))
        End Function
    End Class
End Namespace
